@@ -1,5 +1,5 @@
 package Gtk3;
-$Gtk3::VERSION = '0.016';
+$Gtk3::VERSION = '0.021';
 use strict;
 use warnings;
 use Carp qw/croak/;
@@ -151,8 +151,8 @@ my %_GDK_EVENT_TYPE_TO_PACKAGE = (
   'expose' => 'Expose',
   'motion-notify' => 'Motion',
   'button-press' => 'Button',
-  'button-2press' => 'Button',
-  'button-3press' => 'Button',
+  '2button-press' => 'Button',
+  '3button-press' => 'Button',
   'button-release' => 'Button',
   'key-press' => 'Key',
   'key-release' => 'Key',
@@ -186,6 +186,9 @@ my %_GDK_EVENT_TYPE_TO_PACKAGE = (
   'touch-update' => 'Touch',
   'touch-end' => 'Touch',
   'touch-cancel' => 'Touch',
+  # added in 3.6:
+  'double-button-press' => 'Button',
+  'triple-button-press' => 'Button',
 );
 
 # Make all of the above sub-types inherit from Gtk3::Gdk::Event.
@@ -1422,8 +1425,12 @@ sub Gtk3::Gdk::Pixbuf::new_from_xpm_data {
     $class, $real_data);
 }
 
-# The next three subs might have to change when
-# <https://bugzilla.gnome.org/show_bug.cgi?id=670372> is accepted.
+# Version check for the new annotations described in
+# <https://bugzilla.gnome.org/show_bug.cgi?id=670372>.
+my $HAVE_GDK_PIXBUF_2_31_2 = sub {
+  return (Gtk3::Gdk::PIXBUF_MAJOR () == 2 && Gtk3::Gdk::PIXBUF_MINOR () > 31) ||
+         (Gtk3::Gdk::PIXBUF_MAJOR () == 2 && Gtk3::Gdk::PIXBUF_MINOR () == 31 && Gtk3::Gdk::PIXBUF_MICRO () >= 2);
+};
 
 sub Gtk3::Gdk::Pixbuf::save {
   my ($pixbuf, $filename, $type, @rest) = @_;
@@ -1432,8 +1439,9 @@ sub Gtk3::Gdk::Pixbuf::save {
     croak ("Usage: \$pixbuf->save (\$filename, \$type, \\\@keys, \\\@values)\n",
            " -or-: \$pixbuf->save (\$filename, \$type, \$key1 => \$value1, ...)");
   }
+  my $method = $HAVE_GDK_PIXBUF_2_31_2->() ? 'save' : 'savev';
   Glib::Object::Introspection->invoke (
-    $_GDK_PIXBUF_BASENAME, 'Pixbuf', 'savev',
+    $_GDK_PIXBUF_BASENAME, 'Pixbuf', $method,
     $pixbuf, $filename, $type, $keys, $values);
 }
 
@@ -1444,9 +1452,10 @@ sub Gtk3::Gdk::Pixbuf::save_to_buffer {
     croak ("Usage: \$pixbuf->save_to_buffer (\$type, \\\@keys, \\\@values)\n",
            " -or-: \$pixbuf->save_to_buffer (\$type, \$key1 => \$value1, ...)");
   }
+  my $method = $HAVE_GDK_PIXBUF_2_31_2->() ? 'save_to_buffer' : 'save_to_bufferv';
   my (undef, $buffer) =
     Glib::Object::Introspection->invoke (
-      $_GDK_PIXBUF_BASENAME, 'Pixbuf', 'save_to_bufferv',
+      $_GDK_PIXBUF_BASENAME, 'Pixbuf', $method,
       $pixbuf, $type, $keys, $values);
   return $buffer;
 }
@@ -1458,8 +1467,9 @@ sub Gtk3::Gdk::Pixbuf::save_to_callback {
     croak ("Usage: \$pixbuf->save_to_callback (\$save_func, \$user_data, \$type, \\\@keys, \\\@values)\n",
            " -or-: \$pixbuf->save_to_callback (\$save_func, \$user_data, \$type, \$key1 => \$value1, ...)");
   }
+  my $method = $HAVE_GDK_PIXBUF_2_31_2->() ? 'save_to_callback' : 'save_to_callbackv';
   Glib::Object::Introspection->invoke (
-    $_GDK_PIXBUF_BASENAME, 'Pixbuf', 'save_to_callbackv',
+    $_GDK_PIXBUF_BASENAME, 'Pixbuf', $method,
     $pixbuf, $save_func, $user_data, $type, $keys, $values);
 }
 
@@ -1471,6 +1481,48 @@ sub Pango::Layout::set_text {
     @_ == 3 ? @_ : (@_[0,1], -1)); # wants length in bytes
 }
 
+# - Fixes ------------------------------------------------------------------- #
+
+# Compatibility with perl 5.20 and non-dot locales.  Wrap all functions that
+# might end up calling setlocale() such that POSIX::setlocale() is also called
+# to ensure perl knows about the current locale.  See the discussion in
+# <https://rt.perl.org/Public/Bug/Display.html?id=121930>,
+# <https://rt.perl.org/Public/Bug/Display.html?id=121317>,
+# <https://rt.perl.org/Public/Bug/Display.html?id=120723>.
+if ($^V ge v5.20.0) {
+  require POSIX;
+  no strict 'refs';
+  no warnings 'redefine';
+
+  my $disable_setlocale = 0;
+  *{'Gtk3::disable_setlocale'} = sub {
+    $disable_setlocale = 1;
+    Glib::Object::Introspection->invoke (
+      $_GTK_BASENAME, undef, 'disable_setlocale', @_);
+  };
+
+  # These two already have overrides.
+  foreach my $function (qw/Gtk3::init Gtk3::init_check/) {
+    my $orig = \&{$function};
+    *{$function} = sub {
+      if (!$disable_setlocale) {
+        POSIX::setlocale (POSIX::LC_ALL (), '');
+      }
+      $orig->(@_);
+    };
+  }
+
+  foreach my $function (qw/init_with_args parse_args/) {
+    *{'Gtk3::' . $function} = sub {
+      if (!$disable_setlocale) {
+        POSIX::setlocale (POSIX::LC_ALL (), '');
+      }
+      Glib::Object::Introspection->invoke (
+        $_GTK_BASENAME, undef, $function, @_);
+    };
+  }
+}
+
 # - Helpers ----------------------------------------------------------------- #
 
 sub _common_tree_model_new {
